clear all;
close all;
clc;

t_begin = tic;
%% Initial value

problem = 1;

switch problem
    case 1  % two-level problem, no noise
        % The functions
        f = {@(x)(0.5*((6*x - 2).^2).*sin(12.*x - 4) + 10.*(x - 0.5));
            @(x)( ((6*x - 2).^2).*sin(12.*x - 4) + 10)};
        
        % Input domain
        bnd = stk_hrect([0; 1], {'x'});
        
        % The noise
        noise_std = [0, 0];
        
        % The cost of each level
        cost_lev = [1, 4];
        
        % The critical threshold
        thres = 10;
        
        % Reference
        reg = 2.5;
        range_out = 20;
        
        % Number of points in the initial design
        ni_init = fliplr(cumprod([3, 2]));
        
    case 2  % two-level problem, noised
        % The functions
        f = {@(x)(0.5*((6*x - 2).^2).*sin(12.*x - 4) + 10.*(x - 0.5));
            @(x)( ((6*x - 2).^2).*sin(12.*x - 4) + 10)};
        
        % Input domain
        bnd = stk_hrect([0; 1], {'x'});
        
        % The noise
        noise_std = [1.2; 0.5];
        
        % The cost of each level
        cost_lev = [1, 4];
        
        % The critical threshold
        thres = 10;
        
        % Reference
        reg = 2.5;
        range_out = 20;
        
        % Number of points in the initial design
        ni_init = fliplr(cumprod([6, 2]));
        
    case 3  % three-level problem, no noise
        % The functions
        m = 3;
        a = [1.08; -0.21; 0.40; 0.42;];
        w0 = 1.7*pi;
        f = {@(x) (    a(1)*sin(w0*x) + a(2)*sin(4*w0*x));
            @(x)  (    a(1)*sin(w0*x) + a(2)*sin(4*w0*x) + a(3)*sin(6*w0*x))
            @(x)  (m + a(1)*sin(w0*x) + a(2)*sin(4*w0*x) + a(3)*sin(6*w0*x) + a(4)*sin(8*w0*x))};
        
        % Input domain
        bnd = stk_hrect([-1; 1], {'x'});
        
        % The noise
        noise_std = [0, 0, 0];
        
        % The cost of each level
        cost_lev = [1, 4, 10];
        
        % The critical threshold
        thres = m + 1;
        
        % Reference
        reg = 1.5;
        range_out = 5;
        
        % Number of points in the initial design
        ni_init = fliplr(cumprod([5, 2, 2]));
        
    otherwise
        stk_error('Unknown problem', 'UnknownCase');
end

dim = size(bnd, 2);
nb_lev = length(f);

% Assert cost are sorted
cost_lev = sort(cost_lev);
supp_budget = 9*max(cost_lev);

is_noise = any(noise_std ~= 0);

% Real function
nt = 978;
x  = stk_sampling_regulargrid(nt, dim, bnd);
dx = mean(diff(x));
xt = stk_add_level(x, 1:nb_lev);

zt = stk_dataframe(NaN(nt*nb_lev, 1), {'z'});
for kl = 1:nb_lev
    zt((kl - 1)*nt + (1:nt), 1) = f{kl}(x);
end

idx_nb_lev = (double(xt(:, dim + 1)) == nb_lev);
pt = stk_distrib_normal_cdf(double(zt(idx_nb_lev, 1)), thres, noise_std(nb_lev));

% Number of parameters
nb_sim_par = 100;
% nb_sim_par_surCrit = 1: The MAP parameters

% Parameters of the Haario algorithm
time_no_adapt = 500;
burn_in       = 1000;
delay         = 100;

type_adapt      = 'smooth';
display_waitbar = true;

% Parameter of the Sequential Monte-Carlo sampling
nb_step = 20;

% Integration point (for criterion)
xu = xt(idx_nb_lev, :);
epsilon = 5e-4; % Zero crit

% Maximum number of evalutations
maxEval = 20*dim;
opts_fminbnd = optimset('fminbnd');
opts_fminbnd = optimset(opts_fminbnd, 'Display', 'off', 'MaxFunEvals', maxEval);

% Number of experiments
nb_exp = 20;
nb_algo = nb_lev + 1;

if nb_exp == 1
    tb = tic;
end

if isempty(mfilename('fullpath'))
    grey = @(x)(repmat(min(max(x(:), 0), 1), 1, 3));

    colors = grey(linspace(0.5, 0, nb_lev));
    line = {':', '-.', '--', '-'};
    nl = length(line);
    
    % Plot the functions
    xmark = stk_sampling_regulargrid(20, dim, bnd);
    fig = stk_figure('The functions');
    hold on;
    for ul = 1:nb_lev
        plot(double(x), double(f{ul}(x)), 'Color', colors(ul, :), ...
            'LineStyle', line{nl - nb_lev + ul}, 'LineWidth', 2);
    end
    plot(double(bnd), [thres, thres], 'k--');
    xlim(double(bnd));
    set(get(fig, 'CurrentAxes'), 'FontSize', 14);
    stk_xlabel('x', 'FontSize', 16);
    stk_ylabel('f(x)', 'FontSize', 16);
    stk_title('The functions at each level of fidelity', 'FontSize', 18);
end

%% Style plot
kunc = 1.9599639845; % norminv(97.5/100);
% Noise
color_noise = [0.25, 0.75, 0.3];	% Green-Cyan
style_noise = {'FaceColor', color_noise, 'LineStyle', '-', 'LineWidth', 1,...
    'EdgeColor', color_noise};

% Uncertainty
color_unc = [0.8, 0.8, 0.8];
style_unc = {'FaceColor', color_unc, 'LineStyle', '-', 'LineWidth', 1,...
    'EdgeColor', color_unc};

% Prediction
style_pred  = {'LineStyle', '-', 'LineWidth', 3, 'Color', [0.95 0.25 0.3]};

% Sample paths
color_truth = [0.39, 0.47, 0.64];
style_sim = {'LineStyle', '-',  'LineWidth', 1, 'Color', color_truth};

% Real function
style_truth = {'LineStyle', '--', 'LineWidth', 3, 'Color', color_truth};

% Observations
style_obs   = {'LineStyle', 'none', 'Marker', 'o', 'MarkerSize', 6,...
    'Color', 'k', 'MarkerFaceColor', 'k'};

% Threshold
style_thres =  {'LineStyle', '-.', 'LineWidth', 1, 'Color', 'k'};

% Best reduction
style_best = {'LineStyle', 'none', 'Marker', 's', 'MarkerSize', 6,...
    'Color', 'r', 'MarkerFaceColor', 'none'};

% Limits
style_limit = {'LineStyle', '--', 'LineWidth', 2, 'Color', 'r'};

%% Create a model

% Covariance
cov_ar = stk_covmodel_autoregre(nb_lev, dim, reg);

% Prior of the covariance parameters
if isnan(reg)
    reg = 2.5;
    var_reg = log(5/2);
else
    var_reg = 0;
end

mean_prior = [ones(nb_lev - 1, 1);
    repmat([2*log(range_out/100); -log(diff(bnd)/2)'; log(reg);], nb_lev, 1)];
var_prior = [repmat(2, nb_lev - 1, 1);
    repmat([log(100); repmat(log(10), dim, 1); var_reg;], nb_lev, 1)].^2;

cov_ar.prior = stk_prior_gauss(mean_prior, var_prior);
% Fixed parameters
is_fixed = (var_prior == 0);
    
% Noise
if is_noise
    clnv = 99/100;
    slnv = log(100);
    lnv = stk_noisemodel_multilev(1:nb_lev, NaN(nb_lev, 1), true, 2*log(range_out/100),...
        (slnv^2)*(1 - clnv), (slnv^2)*clnv);
    is_fixed = [is_fixed; (diag(lnv.prior.covariance) == 0)];
else
    lnv = -Inf;
end

% Model
model = stk_model(cov_ar.func, dim + 1);
model.param = cov_ar;
model.lognoisevariance = lnv;
% model.lm = stk_lm_constant();
model.lm = @(x)(stk_cokriging_mean(1:nb_lev, x));


%% Loop of experiment
all_res = struct('results', [], 'xi', [], 'zi', [], 'ni_init', ni_init, 'model', [], 'simul_param', []);
all_res = repmat(all_res, nb_exp, nb_algo);

for ex = 1:nb_exp
    fprintf('Experiment %d/%d\n', ex, nb_exp);
    %% Initial design
    % Observation
    xi_init = stk_sampling_nestedlhs(ni_init, dim, bnd, [], 1:nb_lev);
    
    zi_init = stk_dataframe(NaN(size(xi_init, 1), 1), {'z'});
    for id_l = 1:nb_lev
        idx_il = (xi_init.Level == id_l);
        zi_init(idx_il) = f{id_l}(double(xi_init(idx_il, 1:dim)))...
            + noise_std(id_l)*randn(sum(idx_il), 1);
    end

    % Initial model
    [param_init, lnv_init] = stk_param_init(model, xi_init, zi_init, [bnd, [1; nb_lev]], is_noise);
    if isnumeric(lnv_init) && isinf(lnv_init)
        lnv_init = [];
    end
    [param_init, lnv_init] = stk_param_estim (model, xi_init, zi_init,...
        param_init, lnv_init, @stk_param_mlogpost);
    model.param             = param_init;
    model.lognoisevariance  = lnv_init;
    
    % The fixed parameters
    param_vect = stk_get_optimizable_parameters(model);
    
    param_fixed = NaN(size(param_vect));
    param_fixed(is_fixed) = param_vect(is_fixed);
    % Seed of the adaptive metropolis
    sim_par0 = param_vect(~is_fixed, 1);
    
    % The distribution of parameters
    distrib = @(p)(stk_haario_logpost(model, xi_init, zi_init, p, param_fixed));
    isLogDistrib = true;
    
    % Box of parameters
    [lb_cov, ub_cov] = stk_param_getdefaultbounds(model.param, xi_init, zi_init);
    if is_noise
        [lb_lnv, ub_lnv] = stk_param_getdefaultbounds(model.lognoisevariance,...
            xi_init, zi_init);
        lb = [lb_cov; lb_lnv];
        ub = [ub_cov; ub_lnv];
    else
        lb = lb_cov;
        ub = ub_cov;
    end
    bnd_param = [lb'; ub'];
    bnd_param = bnd_param(:, ~is_fixed);
    
    % Initial covariance of parameters
    cov_par0 = stk_haario_suggest_C0(distrib, bnd_param,...
        sim_par0, isLogDistrib);
    
    % Haario's sampling
    sim_par_init = stk_haario_adaptive_metropolis(distrib, ...
        bnd_param, nb_sim_par, sim_par0, cov_par0,...
        time_no_adapt, burn_in, delay, isLogDistrib,...
        [], type_adapt, display_waitbar);
    
    % Simulated parameters
    simul_param_init = repmat(param_fixed', nb_sim_par, 1);
    simul_param_init(:, ~is_fixed) = sim_par_init;
    
    %% Loop of algorithm
    for algo = 1:nb_algo
        fprintf('Algorithm %d/%d\n', algo, nb_algo);
        % Prepare design
        xi = xi_init;
        zi = zi_init;
        
        ni = ni_init;
        
        cost_init = sum(cost_lev.*ni_init);
        tot_budget = cost_init + supp_budget;
        % Prepare model
        model.param             = param_init;
        model.lognoisevariance  = lnv_init;
        simul_param             = simul_param_init;
        
        % Prepare other informations
        % New points
        xn = [];
        zn = [];
        nn = zeros(1, nb_lev);
        % Predictions
        zp = stk_dataframe(NaN(nb_lev*nt, 3), {'mean', 'var', 'var_obs'});
        pp = stk_dataframe(NaN(nt, 2), {'mean', 'var'});
        
        % Results
        results = stk_dataframe(NaN(0, 4), {'cost', 'error_L2', 'bias', 'variance'});
        time_comp = stk_dataframe(NaN(0, 3), {'smc', 'pred', 'crit'});
        
        while sum((ni + nn).*cost_lev) <= tot_budget
            
            %% Add the new points
            xi = cat(1, xi, xn);
            zi = cat(1, zi, zn);
            ni = ni + nn;
            consumed_budget = sum(ni.*cost_lev) - cost_init;
            fprintf('Cost: %d/%d (%.1f%%)\n',...
                consumed_budget, supp_budget, 100*consumed_budget/supp_budget);
            
            % Add a line to the time computation
            time_comp = cat(1, time_comp, NaN(1, size(time_comp, 2)));
            %% Reestimate parameters
            disp('Updating hyper-parameters...');
            t_update_param = tic;
            
            if is_noise
                [param_opt, lnv_opt] = stk_param_estim (model, xi, zi,...
                    model.param, model.lognoisevariance, @stk_param_mlogpost);
            else
                try
                    [param_opt, lnv_opt] = stk_param_estim (model, xi, zi,...
                        model.param, [], @stk_param_mlogpost);
                catch
                    model.lognoisevariance = log(eps)/2;
                    [param_opt, lnv_opt] = stk_param_estim (model, xi, zi,...
                        model.param, [], @stk_param_mlogpost);
                end
            end
            
            model.param             = param_opt;
            model.lognoisevariance  = lnv_opt;
            
            % Display estimated parameters
            for ul = 1:(nb_lev - 1)
                fprintf('rho_opt[%d] = %g\n', ul, model.param.param.rhoCoeff(ul));
            end
            for pl = 1:nb_lev
                fprintf('Level %d:\n', pl);
                fprintf('    Sigma  = %g\n', model.param.param.(['sig_', num2str(pl)]));
                for kd = 1:dim
                    fprintf('    Rho[%d] = %g\n', kd, model.param.param.(['range_', num2str(pl)]));
                end
                fprintf('    Nu     = %g\n', model.param.param.(['reg_', num2str(pl)]));
                if is_noise
                    fprintf('sqrt(lambda) = %g\n', model.lognoisevariance.param.noistd(pl));
                end
            end
            
            % The vector of hyper-parameters
            vect_estim = stk_get_optimizable_parameters(model);
            param_center = vect_estim(~is_fixed, 1);
            % Move the parameters
            if isempty(xn) % first loop
                SMC_weights = ones(nb_sim_par, 1)/nb_sim_par;
            else
                ni_x = size(xi, 1);
                iniM = 1:(ni_x - 1);
                % SMC - 1: update weights
                SMC_weights = stk_smc_weights(simul_param, model,...
                    xi(iniM, :), zi(iniM, :), xi(ni_x, :), zi(ni_x, :), SMC_weights);
                % SMC - 2: re-sample
                [simul_param, SMC_weights] = stk_smc_sample(simul_param, SMC_weights);
            end
            % SMC - 3: move
            % Select varying parameters
            sim_par = simul_param(:, ~is_fixed);
            % The distribution of parameters
            distrib = @(p)(stk_haario_logpost(model, xi, zi, p, param_fixed));
            
            % Box of parameters
            [lb_cov, ub_cov] = stk_param_getdefaultbounds(param_opt, xi, zi);
            if is_noise
                [lb_lnv, ub_lnv] = stk_param_getdefaultbounds(lnv_opt, xi, zi);
                lb_par = [lb_cov; lb_lnv];
                ub_par = [ub_cov; ub_lnv];
            else
                lb_par = lb_cov;
                ub_par = ub_cov;
            end
            bnd_param = [lb_par'; ub_par'];
            bnd_param = bnd_param(:, ~is_fixed);
            
            % Covariance for parameters
            cov_param = stk_haario_suggest_C0(distrib, bnd_param, param_center, true);
            var_param = diag(cov_param);
            too_close = (var_param./max(var_param) < sqrt(eps));
            var_param(too_close) = diff(bnd_param(:, too_close)).^2*0.01;
            cov_param = diag(var_param);
            
            sim_par = stk_smc_move(sim_par, distrib, nb_step, cov_param);
            % Simulated parameters
            simul_param = repmat(param_fixed', nb_sim_par, 1);
            simul_param(:, ~is_fixed) = sim_par;
            
            time_comp.smc(end) = toc(t_update_param);
            disp('Updating hyper-parameters achived.');
            %% Prediction
            disp('Predicting...');
            t_prediction = tic;
            % Data for prediction
            pred_mean_mean = NaN(nt*nb_lev, 1);
            pred_mean_var  = NaN(nt*nb_lev, 1);
            pred_var_mean  = NaN(nt*nb_lev, 1);
            obs_mean_var   = NaN(nt*nb_lev, 1);

            
            % Data for probability estimations
            fail_mean_mean = NaN(nt, 1);
            fail_mean_var  = NaN(nt, 1);
            fail_var_mean  = NaN(nt, 1);
            
            % Check the MAP parameter
            best_AP = stk_param_mlogpost(model, xi, zi);
            best_cov = param_opt;
            best_lnv = lnv_opt;
            reestimate = false;
            
            for kpar = 1:nb_sim_par
                % 1 - change the parameters
                model = stk_set_optimizable_parameters(model, simul_param(kpar, :));
            
                % Bonus - check posterior
                AP = stk_param_mlogpost(model, xi, zi);
                if AP <= best_AP
                    best_AP = AP;
                    best_cov = model.param;
                    best_lnv = model.lognoisevariance;
                    reestimate = true;
                end
                
                % 2 - compute the prediction, knowing the parameters
                zp_k = stk_predict(model, xi, zi, xt);
                if isnumeric(model.lognoisevariance)
                    % Homoscedastic
                    var_obs_k = exp(model.lognoisevariance);
                else
                    var_obs_k = stk_covmat_noise (model, xt, [], -1, true);
                end
                zp_k_var_obs = zp_k.var + var_obs_k;
                                
                % 3 - the local probability of failure, knowing the parameters
                zp_k_lev = zp_k(idx_nb_lev, :);
                zp_k_lev_var_obs = zp_k_var_obs(idx_nb_lev, :);
                up_k_lev = (zp_k_lev.mean - thres)./sqrt(zp_k_lev_var_obs);
                
                if is_noise
                    rp_k_lev = (zp_k_lev.var)./(zp_k_lev_var_obs);
                    
                    fail_k_mean = stk_distrib_normal_cdf(up_k_lev);
                    fail_k_var  = stk_distrib_bivnorm_cdf([up_k_lev, up_k_lev],...
                        0, 0, 1, 1, rp_k_lev);
                    fail_k_var  = fail_k_var - fail_k_mean.^2;
                else % not noisy model
                    [pp_mean, pq_mean] = stk_distrib_normal_cdf(up_k_lev);
                    fail_k_mean = pp_mean;
                    fail_k_var  = pp_mean.*pq_mean;
                end
                
                % 4 - mean over the parameters (reccursive formulation)
                obs_mean_var = stk_mean_recursive(kpar, obs_mean_var, zp_k_var_obs);
                pred_mean_var = stk_mean_recursive(kpar, pred_mean_var, zp_k.var);
                
                [pred_mean_mean, pred_var_mean] = stk_update_recursive(kpar, pred_mean_mean,...
                    pred_var_mean, zp_k.mean);
                
                % probability of failure
                fail_mean_var = stk_mean_recursive(kpar, fail_mean_var, fail_k_var);
                
                [fail_mean_mean, fail_var_mean] = stk_update_recursive(kpar, fail_mean_mean,...
                    fail_var_mean, fail_k_mean);
            end
            zp.mean     = pred_mean_mean;
            zp.var      = pred_var_mean + pred_mean_var;
            zp.var_obs  = pred_var_mean +  obs_mean_var;
            
            pp.mean	= fail_mean_mean;
            pp.var  = fail_mean_var + fail_var_mean;
            
            % Save results
            results_loop = [sum(ni.*cost_lev),...
                mean((pp.mean - pt).^2),...
                mean(pp.mean) - mean(pt),...
                mean(pp.var)];
            results = cat(1, results, results_loop);
            fprintf('\nCost = %g\n', results_loop(1));
            fprintf('Error   : S_X (^p_n(x) - p*(x))^2 dx   = %g\n', results_loop(2));
            fprintf('Bias    : S_X p_n(x) dx - S_X p*(x) dx = %g\n', results_loop(3));
            fprintf('Variance: S_X Var[p(x)] dx             = %g\n', results_loop(4));
            
            if reestimate
                disp('Reestimation');
                if is_noise
                    [param_opt, lnv_opt] = stk_param_estim(model, xi, zi,...
                        best_cov, best_lnv, @stk_param_mlogpost);
                else
                    try
                        [param_opt, lnv_opt] = stk_param_estim(model, xi, zi,...
                            best_cov, [], @stk_param_mlogpost);
                    catch
                        model.lognoisevariance = log(eps)/2;
                        [param_opt, lnv_opt] = stk_param_estim(model, xi, zi,...
                            best_cov, [], @stk_param_mlogpost);
                    end
                end
            end
            model.param = param_opt;
            model.lognoisevariance = lnv_opt;
            
            time_comp.pred(end) = toc(t_prediction);
            disp('Predicting achieved.');
            %% Compute criteria
            disp('Selecting a new observation...');
            t_select_new_point = tic;
            
            max_red = NaN(nb_lev, 1);
            x_best  = NaN(nb_lev, dim + 1);
            
            switch algo
                case num2cell(1:nb_lev)	% Optimization on a fixed level
                    xc = stk_add_level(x, algo);
                    [~, redc] = stk_sur_pexceedthres(thres, xu, model, xi, zi, xc);
                    
                    % Remove points to close to observations
                    if ~stk_isnoisy(model)
                        idia = (double(xi(:, dim + 1)) == algo);
                        dista = min(stk_dist(xi(idia, 1:dim), x));
                        redc(dista < epsilon) = 0;
                    end
                    
                    % Select the point which maximizes the reduction
                    [~, id_redc] = max(redc);
                    
                    % Optimize a little
                    func_opt = @(x)(stk_sur_pexceedthres(thres, xu,...
                        model, xi, zi, stk_add_level(x, algo)));
                    xini_opt = x(id_redc, :);
                    lower_bnd = double(max(double(bnd(1, :)), xini_opt - dx));
                    upper_bnd = double(min(double(bnd(2, :)), xini_opt + dx));
                   
                    x_opt = fminbnd(func_opt, lower_bnd, upper_bnd, opts_fminbnd);
                    
                    xn = [x_opt, algo];
                    x_best(algo, :) = xn;
                    zn = f{algo}(double(xn(1, 1:dim))) + noise_std(algo)*randn(1);
                    [~, max_red(algo)] = stk_sur_pexceedthres(thres, xu,...
                        model, xi, zi, xn);
                    
                    nn = zeros(1, nb_lev);
                    nn(algo) = 1;
                    
                    % Pseudo reduction
                    red = zeros(nt*nb_lev, 1);
                    idta = (double(xt(:, dim + 1)) == algo);
                    red(idta) = redc;
                    
                case (nb_lev + 1)  % MRSUR
                    [~, red] = stk_sur_pexceedthres(thres, xu, model, xi, zi, xt);
                    
                    msur    = NaN(nb_lev, 1);
                    is_free = false(nb_lev, 1);
                    for kl = 1:nb_lev
                        idtm = (double(xt(:, dim + 1)) == kl);
                        redm = red(idtm);
                        % Remove points which are to close to the
                        % observations
                        if ~stk_isnoisy(model)
                            idim = (double(xi(:, dim + 1)) == kl);
                            
                            distm = min(stk_dist(xi(idim, 1:dim), xt(idtm, 1:dim)));
                            
                            redm(distm < epsilon) = 0;
                            red(idtm) = redm;
                        end
                        
                        % maximal reduction at this level
                        [~, id_red] = max(redm);
                        
                        % Optimize a little
                        func_opt_k = @(x)(stk_sur_pexceedthres(thres, xu,...
                            model, xi, zi, stk_add_level(x, kl)));
                        xini_ko = x(id_red, :);
                        lower_bnd = double(max(double(bnd(1, :)), xini_ko - dx));
                        upper_bnd = double(min(double(bnd(2, :)) , xini_ko + dx));
                        
                        x_opt_k = fminbnd(func_opt_k, lower_bnd, upper_bnd, opts_fminbnd);
                        
                        x_best(kl, :) = [x_opt_k, kl];
                        [~, max_red(kl)] = stk_sur_pexceedthres(thres, xu,...
                            model, xi, zi, x_best(kl, :));
                        
                        msur(kl) = max_red(kl)/cost_lev(kl);
                        % Is the level blocked
                        is_free(kl) = ((consumed_budget + cost_lev(kl)) <= supp_budget);
                    end
                    
                    if all(~is_free)  % if the budget is reached
                        is_free = true(nb_lev, 1);
                        % Select which you want
                    end
                    
                    msur = msur.*double(is_free);   % 0 if the level is not free
                    
                    [~, id_lev] = max(msur);
                    xn = x_best(id_lev, :);
                    zn = f{id_lev}(double(xn(1, 1:dim))) + noise_std(id_lev)*randn(1);
                    
                    nn = zeros(1, nb_lev);
                    nn(id_lev) = 1;
                    
                otherwise
                    stk_error('Unknown algorithm', 'UnknownCase');
            end
            fprintf('\nNew point: (x = %g, t = %d)\n', xn(1, 1), xn(1, 2));
            
            time_comp.crit(end) = toc(t_select_new_point);
            disp('Selecting a new observation achieved.');
            %% Plot real data
            ncol = 3;
            
            if exist('fig', 'var') && ishandle(fig);
                figure(fig);
            else
                fig = stk_figure();
            end
            set(fig, 'name', ['Results with n = [', num2str(ni),...
                '] points (Total = ', num2str(sum(ni)), ')']);
            % Functions
            for kl = 1:nb_lev
                idx_k = (double(xt(:, dim + 1)) == kl);
                
                idi_k = (double(xi(:, dim + 1)) == kl);
                
                zp_k = zp(idx_k, :);
                
                stk_subplot(nb_lev, ncol, ncol*(kl - 1) + 1);
                hold off;
                delta_noise = kunc * sqrt (abs (zp_k.var_obs));
                h_noi = area (double(x), [zp_k.mean - delta_noise, 2 * delta_noise], style_noise{:});
                % Remove the first area object (between 0 and z.mean - delta)
                delete (h_noi(1));  h_noi = h_noi(2);
                hold on;
                % Plot the uncertainty part
                delta = kunc * sqrt (abs (zp_k.var));
                h_ci = area (double(x), [zp_k.mean - delta, 2 * delta], style_unc{:});
                % Remove the first area object (between 0 and z.mean - delta)
                delete (h_ci(1));  h_ci = h_ci(2);
                % The true function
                plot(double(x), double(zt(idx_k, 1)), style_truth{:});
                % The prediction
                plot(double(x), double(zp_k.mean), style_pred{:});
                % Observations
                plot(double(xi(idi_k, 1:dim)), double(zi(idi_k, 1)), style_obs{:});
                if xn(:, dim + 1) == kl
                    hn = plot(double(xn(1, 1)), zn, style_best{:});
                    set(hn, 'MarkerFaceColor', 'r');
                end
                if kl == nb_lev
                    % Threshold
                    plot(double(bnd), repmat(thres, 2, 1), style_thres{:});
                end
                xlim(double(bnd));
                stk_xlabel('x');
                stk_ylabel('z');
                stk_title(sprintf('Level %d', kl));
            end
            
            % Probability of threshold
            
            stk_subplot(1, ncol, 2);
            hold off;
            % Plot the uncertainty part
            delta_pr = kunc * sqrt (abs (pp.var));
            h_vp = area (double(x), [pp.mean - delta_pr, 2 * delta_pr], style_unc{:});
            delete (h_vp(1));  h_vp = h_vp(2);
            hold on;
            % The true function
            plot(double(x), double(pt), style_truth{:});
            % The prediction
            plot(double(x), double(pp.mean), style_pred{:});
            % Threshold
            plot(double(bnd), repmat(0.5, 2, 1), style_thres{:});
            xlim(double(bnd));
            ylim([0; 1]);
            stk_xlabel('x');
            stk_ylabel('p(x)');
            stk_title(sprintf('P(Z[%d] > %g)', nb_lev, thres));
            
            % Reduction of uncertainty
            for kc = 1:nb_lev
                idc_k = (double(xt(:, dim + 1)) == kc);
                
                xc_k = xt(idc_k, 1:dim);
                red_k = red(idc_k, 1);
                                
                stk_subplot(nb_lev, ncol, ncol*(kc - 1) + 3);
                hold off;
                plot(double(xc_k), double(red_k), style_truth{:});
                hold on;
                hm = plot(double(x_best(kc, 1:dim)), max_red(kc), style_best{:});
                if xn(:, dim + 1) == kc
                    set(hm, 'MarkerFaceColor', 'r');
                end
                xlim(double(bnd));
                if max_red(kc) > 0
                    ylim([0; max_red(kc)]);
                end
                stk_xlabel('x');
                stk_ylabel('B(x)');
                stk_title(sprintf('Level %d', kc));
            end
            
            drawnow;
            fprintf('\n');
            
        end
        all_res(ex, algo).results = results;
        all_res(ex, algo).xi = xi;
        all_res(ex, algo).zi = zi;
        all_res(ex, algo).model = model;
        all_res(ex, algo).simul_param = simul_param;
    end
end

%% Save the results
if nb_exp == 1
    % Save the time
    tf = toc(tb);
    
    % List of styles
    color = hsv(nb_algo);
    symb = {'o', 's', 'd', 'v', 'p', 'h', 'x', '+'};
    
    % Figure
    stk_figure('Results');
    for kres = 1:4
        % 3 results + legend
        
        % Ylim
        limits = [0, 0];
        
        % Type of results
        switch kres
            case 1
                field = 'error_L2';
                ylab = '||p(x, tHF) - p*(x, tHF)||';
                tit = 'Error-L2 on the probability function';
                
                fadapt = @sqrt;
                
            case 2
                field = 'bias';
                ylab = '|P - P*|';
                tit = 'Error on the global probability';
                
                fadapt = @abs;
                
            case 3
                field = 'variance';
                field_typ = 'variance';
                ylab = '||Std(p(x, tHF)||';
                tit = 'Variance of the probability function';
                
                fadapt = @sqrt;
                
            otherwise
                field = '';
        end
        
        sub = stk_subplot(2, 2, kres);
        hold on;
        % handles
        hand_algo = NaN(nb_algo, 1);
        % legends
        leg_algo = cell(nb_algo, 1);
        for a = 1:nb_algo
            style = {'LineStyle', '-', 'Color', color(a, :),...
                'MarkerFaceColor', color(a, :), 'LineWidth', 1, ...
                'Marker', symb{a}};
            if kres < 4
                if ~isempty(all_res(1, a).results)
                    cost_a = double(all_res(1, a).results.cost);
                    res_a  = fadapt(all_res(1, a).results.(field));
                    plot(cost_a, res_a, style{:});
                    
                    limits(2) = max(limits(2), max(res_a));
                end
            else
                hand_algo(a) = plot([0; 1], [0; 1], style{:});
                if a > nb_lev
                    leg_algo{a} = 'MRSUR';
                else
                    leg_algo{a} = ['SUR ', num2str(a)];
                end
            end
        end
        if kres < 4
            stk_xlabel('Cost');
            stk_ylabel(ylab);
            stk_title(tit);
            ylim(limits);
            xlim([cost_init, tot_budget]);
        else
            xlim([-2, -1]);
            ylim([-2, -1]);
            
            leg = legend(hand_algo, leg_algo{:});
            set(sub, 'XColor', 'w', 'XTick', [], 'YColor', 'w', 'YTick', []);
            pos_leg = get(leg, 'Position');
            pos_sub = get(sub, 'Position');
            pos_leg([1, 2]) = pos_sub([1, 2]) + (pos_sub([3, 4]) - pos_leg([3, 4]))/2;
            set(leg, 'Position', pos_leg);
        end
    end
    
else
    
    current_file = mfilename('fullpath');
    if isempty(current_file)
        current_file = which('oneDim_seqDoE.m');
    end
    current_folder = fileparts(current_file);
    
    save_file = @(n)(sprintf('%s_%d.mat', fullfile(current_folder, 'AR_save'), round(abs(n))));
    nsave = 0;
    while exist(save_file(nsave), 'file')
        nsave = nsave + 1;
    end
    
    save(save_file(nsave), 'all_res', 'cost_lev');
end

t_end = toc(t_begin);