%% sec43_oscillator_makefig


%% WARNING / suppmat
%
% For the suppmat: remove stuff related to is_lowfi, nb_lowfi...
%


%% How to use
%
% 1) Set the environment variable SEC43_OSCILLATOR_RESDIR to the location
%    of your data.  It can be done within Matlab;  something like that:
%
%    setenv ('SEC43_OSCILLATOR_RESDIR', fullfile (pwd, 'results2'))
%
% 2) Run the script.
%

clear;  close all;  clc;

setenv ('SEC43_OSCILLATOR_RESDIR', fullfile (pwd, 'results6'));


%% Some graphical settings

fig_style = {                         ...
    'InvertHardcopy',      'off',     ...
    'Color',              [1 1 1],    ...
    'PaperPositionMode',  'auto'      };

label_style = {     ...
    'FontSize', 12, ...
    'Color',    'k' };


%% Step 1: Define script parameters

disp ('Step 1: Define script parameters...');

% For a figure similar to the first version of the paper, use c=0.99 in the
% noise model, fixed-level SUR w/ prediction at the HF level (algorithms
% 101--105) et look at the median of pfun_rmse.

% tab = {                       ...
%     101,  ':' ,  [.7 .7 .7];  ...
%     102,  '--',  [.7 .7 .7];  ...
%     103,  '-' ,  [.7 .7 .7];  ...
%     104,  ':' ,  [.0 .0 .0];  ...
%     105,  '--',  [.0 .0 .0];  ...
%     100,  '-' ,  [.0 .0 .0];  };

% For a figure similar to the second version of the paper, use c=0.9 in the
% noise model, fixed-level SUR w/ prediction "current" level (algorithms
% 111--1114 an 1055) et look at the median of pfun_rmse too.

tab = {                       ...
    111,  ':' ,  [.7 .7 .7];  ...
    112,  '--',  [.7 .7 .7];  ...
    113,  '-' ,  [.7 .7 .7];  ...
    114,  ':' ,  [.0 .0 .0];  ...
    105,  '--',  [.0 .0 .0];  ...
    100,  '-' ,  [.0 .0 .0]   };

% Bonus: assess MR-SUR artificially targetting a level which is not the
% actual target (the one where the QoI is defined).

% tab = {                       ...
%     113,  '--',  [.3 .3 .0];  ...
%     123,  '-' ,  [.3 .3 .0];  ...
%     114,  '--',  [.0 .3 .3];  ...
%     124,  '-' ,  [.0 .3 .3];  ...
%     105,  '--',  [.3 .0 .3];  ...
%     100,  '-' ,  [.3 .0 .3]   };

% Batch

% tab = {                       ...
%     113,  '-' ,  [.7 .7 .0];  ...
%     133,  ':' ,  [.7 .7 .0];  ...
%     105,  '--',  [.0 .0 .0];  ...
%     100,  '-' ,  [.0 .0 .0];  ...
%     130,  ':' ,  [.0 .0 .0]   };

nb_algo = size (tab, 1);
nb_sim = 200;


%% Step 2: Load results

disp ('Step 2: Load results...');

ee = cell (nb_algo, 1);

res  = struct (       ...
    'algorithm',  ee, ...
    'pfun_rmse',  ee, ...
    'prob_error', ee, ...
    'cost',       ee, ...
    'dt',         ee, ...
    'is_lowfi',   ee, ...
    'nb_lowfi',   ee  );

cost_min = +inf;
cost_max = -inf;

for idx_sim = 1:nb_sim,  save_dir = sec43_oscillator_savedir (idx_sim);
    
    for a = 1:nb_algo
        
        tmp = load (fullfile (save_dir, ...
            sprintf ('sec43_oscillator_%03d_%04d.mat', tab{a, 1}, idx_sim)));
        
        res(a).algorithm = tmp.res.algorithm;
        res(a).plot_style = {tab{a, 2}, 'Color', tab{a, 3}, 'LineWidth', 2};
        
        res(a).pfun_rmse{idx_sim} = tmp.res.history.pfun_rmse;
        res(a).prob_error{idx_sim} = tmp.res.history.prob_error;
        
        res(a).cost{idx_sim} = tmp.res.history.cost;
        cost_min = min (cost_min, min (res(a).cost{idx_sim}));
        cost_max = max (cost_max, max (res(a).cost{idx_sim}));
        
        n0 = sum (tmp.res.ni_init);
        res(a).dt{idx_sim} = tmp.res.xi.dt(n0+1:end);
        
        res(a).is_lowfi{idx_sim} = (res(a).dt{idx_sim} > 0.1);
        res(a).nb_lowfi(idx_sim) = sum (res(a).is_lowfi{idx_sim});
        
    end
end

dt_list = tmp.res.dt;

clear a cost ee idx_sim pfun_rmse save_dir tmp


%% Step 3: Define cost axis

disp ('Step 3: Define cost axis...');

% Bounds on the cost
cost_lim = stk_hrect ([cost_min; cost_max], {'cost'});

% Cost axis
cost_axis_length = 1001;
cost_axis = stk_sampling_regulargrid (cost_axis_length, 1, cost_lim);


%% Step 4: Resample results on the cost axis

disp ('Step 4: Resample results on the cost axis...');

res_bruts = res;  % Needed below

for a = 1:nb_algo
    
    pfun_rmse = nan (cost_axis_length, nb_sim);
    prob_error = nan (cost_axis_length, nb_sim);
    
    for idx_sim = 1:nb_sim
        cost_ = res(a).cost{idx_sim};
        pfun_rmse_ = res(a).pfun_rmse{idx_sim};
        prob_error_ = res(a).prob_error{idx_sim};
        
        for c = 1:cost_axis_length
            i = find (cost_ <= cost_axis(c), 1, 'last');
            pfun_rmse(c, idx_sim) = pfun_rmse_(i);
            prob_error(c, idx_sim) = prob_error_(i);
        end
    end
    
    res(a).cost = cost_axis;
    res(a).prob_error = prob_error;
    res(a).pfun_rmse = pfun_rmse;
    
end


%% Step 5: Take absolute value of prob_error

disp ('Step 5: Take absolute value of prob_error...');

for a = 1:nb_algo
    res(a).prob_error = abs (res(a).prob_error);
end


%% Step 6: Normalize costs

disp ('Step 6: Normalize costs...');

dt_HF = 0.01;  cost_HF = sec43_dho_cputime (dt_HF);

cost_min = cost_min / cost_HF;
cost_max = cost_max / cost_HF;
cost_lim = cost_lim / cost_HF;

for a = 1:nb_algo
    res(a).cost = res(a).cost / cost_HF;
end


%% Step 7: Compute summary statistics
%
% Note: several metrics are computed---instead of just one---to check that
% the conclusions are not significantly affected by the choice of metric.
%
% It turns out that they are not.
%

disp ('Step 7: Compute summary statistics (mean, median, RMS)...')

metrics = {'prob_error', 'pfun_rmse'};
summary_type = {'avg', 'med', 'rms'};

for j = 1:2,  m = metrics{j};
    
    max_err_simu.(m) = -inf;
    max_err_summ.(m) = -inf;
    
    for a = 1:nb_algo
        
        res(a).(['avg_' m]) = mean (res(a).(m), 2);              % mean
        res(a).(['med_' m]) = median (res(a).(m), 2);            % median
        res(a).(['rms_' m]) = sqrt (mean (res(a).(m) .^ 2, 2));  % RMS
        
        max_err_simu.(m) = max (max_err_simu.(m), max (max (res(a).(m))));
        
        for k = 1:3,  st = summary_type{k};  sn = [st '_' m];
            % Set summary name
            res(a).(sn) = stk_dataframe (res(a).(sn), {sn});
            % Update summary max
            max_err_summ.(m) = max (max_err_summ.(m), max (res(a).(sn)));
        end
    end
end


%% Step 8: Legend for performance comparison

disp ('Step 8: Legend for performance comparison...')

fig_lgd = figure (1);  clf ();  set (fig_lgd, 'Name', 'Legend', fig_style{:});

% Create fake plots
for a = 1:nb_algo
    plot ([0; 1], [0; 1], res(a).plot_style{:}, ...
        'DisplayName', [' ' res(a).algorithm.name] );  hold on;
end

% Hide fake plots, box, etc.
box ('off');  xlim ([-2; -1]);  ylim ([-2; -1]);
set (gca, 'XTick', [], 'YTick', [], 'XColor', 'w', 'YColor', 'w');

% Create legend
lgd = legend ({}, 'box', 'off', 'FontSize', 22, 'Interpreter', 'none');

% Center legend in the figure
pos_leg = get (lgd, 'Position');
pos_leg(1) = 0.5 - pos_leg(3)/2;
pos_leg(2) = 0.5 - pos_leg(4)/2;
set (lgd, 'Position', pos_leg);


%% Step 9: Compare performances

disp ('Step 9: Compare performances...');

for j = 1:2,  m = metrics{j};
    
    for k = 1:3,  st = summary_type{k};
        
        fig_name = sprintf ('sec43_compare_%s_%s', m, st);
        fig_cmp = figure (100 * j + k);  clf (fig_cmp);
        set (fig_cmp, 'Name', fig_name, fig_style{:});
        
        sn = [st '_' m];
        for a = 1:nb_algo
            plot (res(a).cost, res(a).(sn), res(a).plot_style{:});
            hold on;
        end
        
        box ('off');  ymax = max_err_summ.(m) * 1.05;
        xlim (cost_lim);  xlabel ('cost', label_style{:});
        ylim ([0 ymax]);  ylabel (sn, label_style{:});
        
    end
end


%% Step 10: Detailed plots

disp ('Step 10: Detailed plots...')

for j = 1:2
    for a = 1:nb_algo
        
        m = metrics{j};
        figure (10 * j + a);  hold off;
        
        cc = res(a).cost;
        mm = res(a).(m);
        
        h = plot (cc, mm, 'Color', 0.8 * [1 1 1]);
        h(1).DisplayName = 'Simulations';
        for i = 2:nb_sim
            h(i).HandleVisibility = 'off';
        end
        hold on;
        
        if (tab{a, 1} == 100) || (tab{a, 1} == 130)
            b = (res(a).nb_lowfi >= 10);
            h = plot (double (cc), mm(:,b), 'Color', 0.8 * [1 0.5 0.5]);
            h(1).DisplayName = 'Simulations (nb lowfi > 30)';
            for i = 2:(length (h))
                h(i).HandleVisibility = 'off';
            end
        end
        
        plot (cc, res(a).(['avg_' m]), ...
            'Color', 'k', 'LineWidth', 2, 'LineStyle', '-', 'DisplayName', 'Average');
        plot (cc, res(a).(['med_' m]), ...
            'Color', 'k', 'LineWidth', 2, 'LineStyle', '--', 'DisplayName', 'Median');
        plot (cc, res(a).(['rms_' m]), ...
            'Color', 'k', 'LineWidth', 2, 'LineStyle', '-.', 'DisplayName', 'RMS');
        
        legend ();
        title (res(a).algorithm.name, 'Interpreter', 'none');
        xlim (cost_lim);              xlabel ('cost', label_style{:});
        ylim ([0 max_err_simu.(m)]);  ylabel (m, label_style{:});
        
    end
end


%% Step 11: Investigate MR-SUR bsize=1 -> visualize selected levels

a_MRSUR = find ([tab{:, 1}] == 100);
if ~ isempty (a_MRSUR),  res0 = res_bruts(a_MRSUR);
    
    figure (999);  nr = 3;  nc = 3;
    
    for k = 1:(nr * nc),  subplot (nr, nc, k);
        
        idx_sim = k;
        
        c = res0.cost{idx_sim}(1:end-1) / cost_HF;
        dt = res0.dt{idx_sim};
        
        for dt_ = dt_list
            plot ([min(c) max(c)], dt_ * [1 1], '--', 'Color', 0.3 * [1 1 1]);  hold on;
        end
        
        plot (c, dt, 'ko', 'MarkerFaceColor', 0.7 * [1 1 1]);  xlim (cost_lim);
        
        ylim ([0 1.1 * max(dt)]);
        title (sprintf ('idx sim = %d', idx_sim));
        xlabel ('cost', label_style{:});  ylabel ('dt', label_style{:});
        
    end
    
end


%% Step 12: Investigate MR-SUR bsize=5 -> visualize selected levels

a_MRSUR = find ([tab{:, 1}] == 130);
if ~ isempty (a_MRSUR),  res0 = res_bruts(a_MRSUR);
    
    figure (1000);  nr = 3;  nc = 3;
    
    for k = 1:(nr * nc),  subplot (nr, nc, k);
        
        idx_sim = k;
        
        c = res0.cost{idx_sim}(1:end-1) / cost_HF;
        dt = res0.dt{idx_sim};
        dt = dt(1:5:end);  % one per batch
        
        for dt_ = dt_list
            plot ([min(c) max(c)], dt_ * [1 1], '--', 'Color', 0.3 * [1 1 1]);  hold on;
        end
        
        plot (c, dt, 'ko', 'MarkerFaceColor', 0.7 * [1 1 1]);  xlim (cost_lim);
        
        ylim ([0 1.1 * max(dt)]);
        title (sprintf ('idx sim = %d', idx_sim));
        xlabel ('cost', label_style{:});  ylabel ('dt', label_style{:});
        
    end
    
end


%% Remarks, TODO, etc.
%
% Detailed plots:
%
% * Implement density plots ???
% * Include these plots in the suppmat ?
%
% TODO
%
% * Explain why the intriguing phenomonon seen on pfun_rmse curves does
%   not appear on prob_error curves...
% * Understand why MR-SUR seems to stall after C=14 approximately
%   (or perhaps is it juste VERY slow after this point ?)
% * Extend simulations at delta = delta_REF = 0.01 to see if it converges.
%


%% Save

clear lgd fig_lgd fig_cmp
[~, ss] = fileparts (getenv ('SEC43_OSCILLATOR_RESDIR'));
save (sprintf('temp_mat/sec43_oscillator_makefig_%s.mat', ss));

