% GAUSS_DISTRIB returns a normal distribution to test Haario algorithm
% 
% CALL: TPG = gauss_distrib(X, MU, SIG2)
function tpG = gauss_distrib(x, mu, SIG2)

persistent SIG2_0 DET ISIG2
% Save costly computable data

if isempty(SIG2_0) || ~isequal(SIG2, SIG2_0)
    DET = det(SIG2);	% determinant of the covariance
    ISIG2 = inv(SIG2);	% inverse of the covariance
    SIG2_0 = SIG2;      % covariance
end

% Constant coefficient
coe = 1/sqrt( ((2*pi)^size(SIG2, 1))*DET );

% (x - m)'*S^{-1}*(x - m)
logDPX = NaN(size(x, 1), 1);
mu = mu(:);
DX = (x' - repmat(mu, 1, size(x, 1)));  % x - m

for k = 1:size(x, 1)    % for each input x
    logDPX(k) = -0.5*DX(:, k)'*ISIG2*DX(:, k);
end

tpG = coe * exp(logDPX);
end

