% This file proposes an example of simulation and prediction with a
% Sequential Monte Carlo algorithm.

config_init;    % Function, observations, configurations

% Model with prior and MaP parameters
model = set_model_init(covariance, xi, zi, bnd, range_z, is_noise);

%% Samples according to the posterior distribution using Sequential Monte Carlo

% Rules the behavior of the algorithm
t0 = 10;                        % length of the chain
nthres = floor(0.9*nb_param);	% Threshold of effective number



% Sampling
ts = tic;
[sim_param, weights] = sampling_smc(nb_param, model, xi, zi, nthres, t0);
time_simulate = toc(ts);
fprintf('Time for simulating hyper-parameters: %.0f s = %d min %.0f s\n',...
    time_simulate, floor(time_simulate/60), time_simulate - 60*floor(time_simulate/60) );

%% Plot density of parameters
fig_par = plot_density_parameters(sim_param, model, sig_noise, nd);

%% Make predictions
% Predictions taking into account on simulated parameters
tp = tic;
zp = stk_smc_predict(sim_param, weights, model, xi, zi, xt);
time_predict = toc(tp);
fprintf('Time for predicting: %.2f s\n', time_predict);

%% Figure
stk_figure('Results with Sequential Monte Carlo');
plot1d_noise ([], xi, zi, xt, zt, zp);
% Bounds
xlim(bnd);
% Labels
stk_xlabel('x');
stk_ylabel('Z');
stk_title('Prediction using Sequential Monte Carlo');