% SAMPLING_HAARIO proposes an example of script to sample parameters
%
% CALL: sim_param = sampling_haario(nb_param, model, xi, zi, t0, t1, t2, updating_formula)
function [sim_param, fig] = sampling_haario(nb_param, model, xi, zi, t0, t1, t2, updating_formula)

%% Default values

% Rules the behavior of the algorithm
if nargin < 5 || isempty(t0)
    t0 = 200;   % burn-in 'without adaptation'
end

if nargin < 6 || isempty(t1)
    t1 = 500;   % burn-in 'with adaptation'
end

if nargin < 7 || isempty(t2)
    t2 = 20;    % Delay between two returned parameters
end

if nargin < 8 || isempty(updating_formula)
    updating_formula = 'hard';  % Available: 'hard', 'smooth' or 'hastings'
end

%% Sampling
distrib = @(p)(-stk_param_relik_val(model, xi, zi, p));
isLogDistrib = true;

% Bounds of parameters
[lb_cov, ub_cov] = stk_param_getdefaultbounds(model.covariance_type, model.param, xi, zi);
if stk_isnoisy(model) && isscalar(model.lognoisevariance)
    lnv_init = model.lognoisevariance;
    
    lb_lnv = min(log(eps)    , lnv_init  - 0.5);
    ub_lnv = max(log(var(zi)), lnv_init) + 0.5;
    
else
    lb_lnv = [];
    ub_lnv = [];
    lnv_init = [];
end

bnd_param = [lb_cov', lb_lnv'; ub_cov', ub_lnv'];

% First parameter
p_init = [model.param(:); lnv_init];

% Find an initial value for the covariance of the instrumental distribution
C0 = stk_haario_suggest_C0(distrib, bnd_param, p_init, isLogDistrib);


epsilon = [];   % use default values
display_progress = true;

% Sampling
[sim_param, acc] = stk_haario_adaptive_metropolis(distrib, bnd_param, nb_param,...
    p_init, C0, t0, t1, t2, isLogDistrib, epsilon, updating_formula, display_progress);

% Compute the autocorrelation of the chains
autocorr = stk_haario_autocorr(sim_param, 1);
max_corr1 = max(abs(autocorr(2, :)));
% Display the measures of quality of the sampling
display(sprintf('Acceptance ratio = %.1f%%; maximal auto-correlation = %.1f%%.',...
    100*acc, 100*max_corr1));

% Plot auto-correlation figures
fig = stk_haario_figure(sim_param, 'corr_param', model);

end

