% STK_PROTOMODEL is an abstract constructor of the protomodel class.
%
% STK_PROTOMODEL has at least 3 descendant classes :
%   - stk_covmodel, which defines the covariance of the Gaussian process
%   - stk_meanmodel, which defines the mean of the Gaussian process
%   - stk_noisemodel, which defines the noise variance of the Gaussian
%   process.
%
% A STK_PROTOMODEL is defined by :
%   - a function, which can be a string, a function_handle or a class.
%   - a parameter, which can be a vector or a stk_param
%   - a prior log-distribution, which can be a structure, a stk_prior or let
%   empty.
%
% You are not supposed to launch this constructor, or create a
% STK_PROTOMODEL object.
function promo = stk_protomodel(func, param, prior)

switch nargin
    case 0
        func = '';
        param = [];
        prior = [];
    case 1
        param = [];
        prior = [];
    case 2
        prior = [];
end

%% Function
if nargin > 0 && isempty(func)
    stk_error('''func'' is empty. You forget to define your function.',...
        'EmptyInputArgs');
elseif isa(func, 'function_handle')
    func = func2str(func);  % convert to string
end

% no particular check, for now

%% Parameters
if ~isempty(param) && ~isnumeric(param) && ~isa(param, 'stk_param')
    % tolerate: empty parameters; numeric parameters; and stk_param classes.
    stk_error(['A parameter of a stk_protomodel must be',...
        ' a numeric or a stk_param.'], 'InvalidArgument');
end

% matrix are tolerate for now

%% Prior
if ~isempty(prior) && ~isstruct(prior) && ~isa(prior, 'stk_prior')
    % tolerate: empty priors; structure; and stk_prior classes.
    stk_error(['A prior log-distribution of a stk_protomodel must be',...
        ' a structure or a stk_prior.'], 'InvalidArgument');
end
% Prior can be let empty (case without prior log-distribution).


%% Build object

promo = struct('func', func, 'param', param, 'prior', prior);
promo = class(promo, 'stk_protomodel');
end

