% FEVAL [overload base]
%
% CALL : kov = feval (func, covmo, xi, yi)
%
% CALL : kov = feval (func, covmo, xi, yi, diff)
%
% CALL : kov = feval (func, covmo, xi, yi, diff, pairwise)
%
% Build the covariance matrix (or Gram matrix) of the covariance model
%
% - covmo : a covariance model;
% - xi, yi : some points where covariance must be compute
% - diff (default value : -1) : ask for a derivative covariace according to
% the diff-th parameter of the covariance model;
% - pairwise : (default value : false) a boolean inidcating if xi and yi
% must be considered pairwise or not.
%
% If size(xi) = [n, d] and size(yi) = [m, d]
%
% then  If pairwise
%           n ~= m ==> error
%           n == m => size(kov) = [n, 1]
%       Else
%           size(kov) = [n, m]
%       End

function kov = feval (func, covmo, xi, yi, diff, pairwise)

% We should land here when covmo is an stk_covmodel object.  Check.
assert (isa (covmo, 'stk_covmodel'));

% pairwise : false by default
if (nargin < 5) || (isempty (pairwise))
    pairwise = false;
end

% diff: -1 by default
if (nargin < 4) || (isempty (diff))
    diff = -1;
end

assert_func_consistency (covmo, func);

% Compute covariance matrix
kov = feval (func, get (covmo, 'param'), xi, yi, diff, pairwise);

end
