% CALL : logsigma2 = stk_get_log_sigma2(covmo)
%
% This function should return a value of log(s^2), where s^2 is the
% variance at 0 of the covariance.
% If the covariance k is stationnary (k(x, y) = c(x - y)), then s^2 = c(0).
% Otherwise, s^2 is a kind of "mean value" of k(x, x).
%
% Remark : if param does not have fields 'logsig2' or 'logsig2', then the
% returned value is 'NaN'.
function logsigma2 = stk_get_log_sigma2(covmo)

param = get(covmo, 'param');
if isfield(param, 'logsig2')
    % if there is a field named 'logsig2' (what we are looking for)
    logsigma2 = get(param, 'logsig2');
    
elseif isfield(param, 'sig2')
    % else, if there is a field named 'sig2' ==> Calculation
    logsigma2 = log(get(param, 'logsig2'));
    
else % If nothing was found,...
    logsigma2 = NaN;    %... return nothing
end

end

