% STK_COVMODEL_AUTOREGRE  computes an auto-regressive multi-fidelity
% covariance model
%
% CALL: COVMO_AR_FID = stk_covmodel_autoregre(NB_LEV, DIM, REG)
function covmo_ar_fid = stk_covmodel_autoregre(nb_lev, dim, reg)

if nargin > 3
    stk_error('Too many input arguments', 'TooManyInputArgs');
end
%% Default values
if nargin < 3 || isempty(reg)
    reg = NaN;
else
    if reg < 0
        stk_error('Regularity must be a positive parameter.', 'InvalidArgument');
    end
end

if nargin < 2 || isempty(dim)
    dim = 1;
else
    dim = ceil(dim);
    if dim < 1
        stk_error('Dimension must be a strictly positive parameter.', 'InvalidArgument');
    end
end

if nargin < 1 || isempty(nb_lev)
    nb_lev = 2;
else
    nb_lev = ceil(nb_lev);
    if nb_lev < 2
        stk_error('Number of levels must be greater or equal to 2.', 'InvalidArgument');
    end
end


%% Function
func = 'stk_arcov_matern';

%% Parameter
mat_param = stk_param_matern_aniso(NaN, NaN(dim, 1), log(reg));
params = repmat({mat_param}, nb_lev, 1);

rhoCoeff = NaN(nb_lev - 1, 1);
param = stk_param_ar_cov(rhoCoeff, params);

%% Prior
prior = [];

%% Build the model
covmo_ar_fid = stk_covmodel(func, param, prior);
covmo_ar_fid = class(struct(), 'stk_covmodel_autoregre', covmo_ar_fid);
end

