% CALL : covmo_mataniso = stk_covmodel_maternaniso(dim, reg)
%
% Create a covariance model with a Matern anisotropic covariance function.
%
% - dim (default value : 1) : dimension of the model;
% - reg (default value : NaN) : the regularity of the function.
function covmo_mataniso = stk_covmodel_maternaniso(dim, reg)

if nargin > 2
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

%% Default values
if nargin < 2 || isempty(reg)
    reg = NaN;
else
    if reg < 0
        stk_error('Regularity must be a positive parameter.', 'InvalidArgument');
    end
end

if nargin < 1 || isempty(dim)
    dim = 1;
else
    dim = ceil(dim);
    if dim < 1
        stk_error('Dimension must be a strictly positive parameter.', 'InvalidArgument');
    end
end

%% function and parameters
if isinf(reg)
    func = 'stk_gausscov_aniso';
    param = stk_param_aniso(NaN, NaN(dim, 1));
else
    func = 'stk_materncov_aniso';
    param = stk_param_matern_aniso(NaN, NaN(dim, 1), log(reg));
end

%% Prior
prior = [];

%% Create the covariance model.
covmo_mataniso = stk_covmodel(func, param, prior);
covmo_mataniso = class(struct(), 'stk_covmodel_maternaniso', covmo_mataniso);
end

