% CALL : [name_fields, numb] = find_field(param, diff)
%
% This function should be used when derivative with respect to a parameter
% want to be caculated. This function takes a parameter and a integer
% vector "diff", and return the field which must be derivative, and the
% number inside the field which must be derivative.
%
% - param : an stk_parameter
% - diff : an integer between 1 and length(param)
%
%
% * name_fields : the field name of param which must be derivate
% * numb : an integer between 1 and length( get(param,
% name_fields_diff) ), indicating which number inside the field must be
% derivate.
%
% Ex : dim = 4; param = stk_param_matern_aniso([], NaN(dim, 1), [])
%   
%   param = 
% 
% logsig2 : NaN
%  logacc : NaN
%           NaN
%           NaN
%           NaN
%  logreg : NaN
%
%   find_field(param, 1)            ==> ['logsig2', 1]
%   find_field(param, 3)            ==> ['logacc', 2]
%   find_field(param, [2, 4; 5, 6]) ==> ['logacc', 1
%                                        'logacc', 4
%                                        'logacc', 3
%                                        'logreg', 1] <- warning : 2
%                                        vectors
%   find_field(param, 7)            ==> error !
%   find_field(param, 0)            ==> error !
%   find_field(param, 0.5)          ==> ['logsig2', 1] 
function [name_fields, numb] = find_field(param, diff)

diff = diff(:); % assert vector
%% Check value of diff
if any(diff < 1)
    stk_error('The ''diff'' parameter must be a positive non-zero integer.',...
        'InvalidArgument');
end

%% Get back field names and sizes
[name_optim_fields, cumlength_optim_fields, total_lenght] = optimizable_fields(param);

if any(diff > total_lenght)
    stk_error(['The ''diff'' parameter must be lower than the total number',...
        ' of parameters (here ', num2str(total_lenght),').'], 'InvalidArgument');
end

%% Find the field and the associated number
num_diff = numel(diff);
ind_diff = NaN(size(diff));
for k = 1:num_diff
    ind_diff(k) = find(diff(k) <= cumlength_optim_fields, 1);
    % support decimal number for diff
end
ind_diff = ind_diff(:);
name_fields = name_optim_fields(ind_diff - 1);
if num_diff == 1
    name_fields = name_fields{1};
end
numb = ceil(diff - cumlength_optim_fields(ind_diff - 1));

end

