% CALL : par_aniso = set(par_aniso, propname, value)
% [overload stk_param function]
function par_aniso = set(par_aniso, propname, value)

switch propname
    case 'logsig2'
        % Assert class
        if ~isnumeric(value)
            try
                value = double(value);
            catch
                stk_error('''logsig2'' must be a double.', 'InvalidType');
            end
        end
        % Assert size
        if numel(value) > 1
            stk_error('''logsig2'' must have a unique value.', 'IncorrectSize');
        end
        % Change value
        par_aniso.logsig2 = value;
        
    case 'logacc'
        % Assert class
        if ~isnumeric(value)
            try
                value = double(value);
            catch
                stk_error('''logacc'' must be a double.', 'InvalidType');
            end
        end
        value = value(:);           % Assert vector
        par_aniso.logacc = value;	% Change value
        
        
        %% Other supported changes
    case 'sig2'
        par_aniso = set(par_aniso, 'logsig2', reallog(value));
        % reallog to ensure that input is positive
    case 'acc'
        par_aniso = set(par_aniso, 'logacc', reallog(value));
    case 'logsig'
        par_aniso = set(par_aniso, 'logsig2', 2*value);
    case 'sig'
        par_aniso = set(par_aniso, 'logsig2', 2*reallog(value));
        % sqrt{exp(lv)} = exp(0.5*lv)
    case 'logrange'
        par_aniso = set(par_aniso, 'logacc', -value);
    case 'range'
        par_aniso = set(par_aniso, 'logacc', -log(value));
        % 1/exp(la) = exp(-la)
        
    otherwise
        stk_error(['An anisotropic parameter does not have any property',...
            ' whose name is ', propname, '.'], 'UnknownField');
end
end

