% CALL : [lb, ub] = stk_param_getdefaultbounds(ar_cov, xi, zi)
%
% Get the lower bound and upper bound of optimization for a parameter.
% lb <= param <= ub.
% lb and ub are vector with the same length as param.
function [lb, ub] = stk_param_getdefaultbounds (ar_cov, xi, zi)

if nargin > 3
    stk_error('Too many input arguments.', 'TooManyInputArgs');
end

%% Bounds

xi = double(xi);
zi = double(zi);
dim = size(xi, 2);

if size(zi, 2) ~= 1
    stk_error('zi is not a vector', 'InvalidArgument')
end

if size(zi, 1) ~= size(xi, 1)
    stk_error('zi and xi have not the same lenght.', 'InvalidArgument');
end

xi_x = xi(:, 1:(dim - 1));
xi_t = xi(:, dim);
nb_lev = number_levels(ar_cov);

if any(xi_t < 1) || any(xi_t > nb_lev) || any(xi_t ~= floor(xi_t))
    stk_error('Levels of x must be integer between 1 and the number of levels', 'InvalidArgument');
end

% List of parameters
params = ar_cov.params;

% Bounds of the parameters
lb_p = [];
ub_p = [];

% Bounds of the rhoCoeff
lb_r = ones(nb_lev - 1, 1);
ub_r = ones(nb_lev - 1, 1);

for kl = 1:nb_lev
    indKl = (xi_t == kl);
    xi_xkl = xi_x(indKl, :);
    if ~isempty(xi_xkl)
        %% Corrections
        zi_kl = zi(indKl, :);
        [lb_p_k, ub_p_k] = stk_param_getdefaultbounds(params{kl}, xi_xkl, zi_kl);
        
        lb_p = cat(1, lb_p, lb_p_k);
        ub_p = cat(1, ub_p, ub_p_k);
        
        
        %% Scales
        if kl < nb_lev
            indKlP1 = (xi_t == (kl + 1));
            xi_xklP1 = xi_x(indKlP1, :);
            if isempty(xi_xklP1) == false
                zi_klP1 = zi(indKlP1, :);
                ratio = (1./zi_kl)*zi_klP1';
                ratio = ratio(:);
                
                percent = 0.2;
                quant = quantile(ratio, [percent, 1 - percent]);
                lb_r(kl) = quant(1);
                ub_r(kl) = quant(2);
            end
        end
        
    else
        param_k = params{kl};
        lb_p = cat(1, lb_p, param_k(:));
        ub_p = cat(1, ub_p, param_k(:));
    end
end

lb = [lb_r; lb_p];
ub = [ub_r; ub_p];

end


