% CALL : cstlnv = stk_param_cstlnv(lognoivar)
%
% Build a constant log-noise variance parameters.
% This parameter should be used for constant log-noise variances.
% Can contain several values for several cases.
% - lognoivar : values of the log-noise variances.
function cstlnv = stk_param_cstlnv(lognoivar)


%% Values of log-noise variance
if nargin < 1 || isempty(lognoivar)
    lognoivar = NaN;    % Default value : no value
else
    % Log-noise variance must be a double.
    if ~isnumeric(lognoivar)
        try
            lognoivar = double(lognoivar);
        catch
            stk_error('''lognoivar'' must be double.', 'InvalidType');
        end
    end
    
    lognoivar = lognoivar(:); % convert in vector
end


%% Build the isometric parameter
cstlnv = struct('lognoivar', lognoivar);
cstlnv = class(cstlnv, 'stk_param_cstlnv', stk_param());

end

