% CALL : mf_cov = set(mf_cov, propname, value)
% [overload stk_param function]
function mf_cov = set(mf_cov, propname, value)

switch propname
    case {'param_Ideal', 'param_Origin', 'paramI', 'paramO', 'param0'}
        if ~isa(value, 'stk_param_matern_aniso')
            stk_error(['The covariance of the ideal part must be described ',...
                'by an anistropic Matern covariance.'], 'InvalidArgument');
        end
        dim = length(value.logacc);
        if dim ~= length(get(mf_cov, 'logacc0'));
            stk_error(['The parameter of the ideal part does not have the',...
                ' same length as the old one.'], 'IncorrectSize');
        end
        mf_cov.param_Ideal = value;
        
    case {'param_Error', 'paramE'}
        if ~isa(value, 'stk_param_matern_aniso')
            stk_error(['The covariance of the error part must be described ',...
                'by an anistropic Matern covariance.'], 'InvalidArgument');
        end
        dim = length(value.logacc);
        if dim ~= length(get(mf_cov, 'logaccE'));
            stk_error(['The parameter of the error part does not have the',...
                ' same length as the old one.'], 'IncorrectSize');
        end
        mf_cov.param_Error = value;
        
    case {'param_Level', 'paramL', 'paramT'}
        if ~isa(value, 'stk_param_nonstatvar_cov')
            stk_error(['The covariance of the level evolution must be described ',...
                'by a non-stationnary covariance.'], 'InvalidArgument');
        end
        
        mf_cov.param_Level = value;
        
        %% Other value you are allowed to get directly
    case 'logESR'
        % ESR = s2_E/s2_0 <=> ln(ESR) = ln(s2_E) - ln(s2_0) <=> ln(s2_E) =
        % ln(ESR) + ln(s2_0)
        mf_cov = set(mf_cov, 'logsig2E',       value  + get(mf_cov, 'logsig20'));
    case 'ESR'
        mf_cov = set(mf_cov, 'logsig2E',   log(value) + get(mf_cov, 'logsig20'));
    case 'logESR12'
        % 2*log(ESR^(1/2)) = log(ESR)
        mf_cov = set(mf_cov, 'logsig2E', 2*    value  + get(mf_cov, 'logsig20'));
    case 'ESR12'
        mf_cov = set(mf_cov, 'logsig2E', 2*log(value) + get(mf_cov, 'logsig20'));
        
    case 'logdegr'
        mf_cov = set(mf_cov, 'logdegrL', value);
    case 'degr'
        mf_cov = set(mf_cov, 'degrL', value);
        
    case 'intercept'
        mf_cov = set(mf_cov, 'interceptL', value);
        
        %% Other propositions : change a property of one of the 3 parameters
    otherwise
        ind_prop = propname(end);   % The last letter of the propname
        propname = propname(1:(end - 1));
        switch ind_prop
            % indicate which parameters to explore
            case {'I', 'O', '0'}
                mf_cov.param_Ideal = set(mf_cov.param_Ideal, propname, value);
            case 'E'
                mf_cov.param_Error = set(mf_cov.param_Error, propname, value);
            case {'T', 'L'}
                if strcmp(propname, 'logsig2') || strcmp(propname, 'sig2')...
                        || strcmp(propname, 'sig') || strcmp(propname, 'logsig')
                    stk_error(['You are not allowed to modify the variance at 0,',...
                        ' of the level parameter.'], 'ReadOnlyField')
                else
                    mf_cov.param_Level = set(mf_cov.param_Level, propname, value);
                end
            otherwise
                % default
                mf_cov.param_Ideal = set(mf_cov.param_Ideal, [propname, ind_prop], value);
        end
            
end

end

