% CALL : nonstat_cov = set(nonstat_cov, propname, value)
% [overload stk_param function]
function nonstat_cov = set(nonstat_cov, propname, value)

switch propname
    case 'param'
        % Assert class
        if ~isa(value, 'stk_param')
            stk_error('param must be a stk_param object.', 'InvalidArgumebt');
        end
        % Change value
        nonstat_cov.param = value;
        
    case 'logdegr'
        % Assert class
        if ~isnumeric(value)
            try
                value = double(value);
            catch
                stk_error('''logdegr'' must be a double.', 'InvalidType');
            end
        end
        % Assert size
        if numel(value) > 1
            stk_error('''logdegr'' must have a unique value.', 'IncorrectSize');
        end
        % Change value
        nonstat_cov.logdegr = value;
        
    case 'intercept'
        % Assert class
        if ~isnumeric(value)
            try
                value = double(value);
            catch
                stk_error('''intercept'' must be a double.', 'InvalidType');
            end
        end
        % Assert size
        if numel(value) > 1
            stk_error('''intercept'' must have a unique value.', 'IncorrectSize');
        end
        % Change value
        nonstat_cov.intercept = value;
        
        %% Other supported changes
    case 'degr'
        nonstat_cov = set(nonstat_cov, 'logdegr', reallog(value));
        % reallog to ensure that input is positive
        
    otherwise
        % else, try to change the parameter
        param = set(nonstat_cov.param, propname, value);
        nonstat_cov = set(nonstat_cov, 'param', param);
end

end