% CALL : gauss_lp = stk_constrain_param(gauss_lp, ind)
%
% CALL : gauss_lp = stk_constrain_param(gauss_lp, ind, m_ind)
%
% Change the covariance of the Gaussian log-prior in order to impose the
% ind-th parameters was equals to m_ind.
%
% - gauss_lp : a stk_prior_gauss;
% - ind : an index between 1 and dim(gauss_lp);
% - m_ind (optionnal) : if provided, the value of param(ind-th).
function gauss_lp = stk_constrain_param(gauss_lp, ind, m_ind)

if nargin > 3
    stk_error('Too many input arguments.', 'TooManyInputArgs');
end
%%
ind = ind(:);
dim = dimension(gauss_lp);
if any(ind < 1) || any(ind > dim)
    stk_error(['Index must be between 1 and the dimension of',...
        ' the Gaussian log-prior.'], 'InvalidArgument');
end

%%
len = length(ind);
one_dim = 1:dim;
not_ind = one_dim(~ismember(one_dim, ind));

P = gauss_lp.eigenvect;
v = gauss_lp.eigenvals;
S = P*diag(v)*P';

if isequal(P(not_ind, ind), zeros(dim - len, len)) ...
        && isequal(P(ind, not_ind), zeros(len, dim - len))
    gauss_lp.eigenvals(ind) = 0;
    gauss_lp.eigenvect(ind, ind) = eye(len);
elseif all(~isinf(S(not_ind, not_ind))) && all(~isnan(S(not_ind, not_ind)))
    S(:, ind) = 0;  % Impose Var[X(ind)] = 0
    S(ind, :) = 0;
    
    gauss_lp = set(gauss_lp, 'covariance', S);
    if nargin > 2
        gauss_lp.mean(ind) = m_ind;
    end
else
    stk_error('Case with infinite correlation is not treated.', 'AlgorithmFailure');
end
end

