% CALL : minus_logprior = stk_mlogprior(gauss_lp, param)
%
% CALL : [minus_logprior, dmlog_prior_dpar] = stk_mlogprior(gauss_lp, param)
%
% [Overload stk_prior function]
% By definition, this function returns :
% minus_logprior = 0.5*(param - mean)'*[eigvect*(diag(1./eigvals))*eigvect']*(param - mean)'
%           + 0.5*dim*log(2*pi) + 0.5*log(prod(eigvals))
%
% dmlog_prior_dpar = [eigvect*(diag(1./eigvals))*eigvect']*(param - mean)'
function [mlog_prior, dmlog_prior_dpar] = stk_mlogprior(gauss_lp, param)

if nargin > 2
    stk_error('Too many input arguments.', 'TooManyInputArgs');
end

%% Get parameter
vect_param = stk_get_optimizable_parameters(param); % vector of parameters
dim = dimension(gauss_lp);
if length(vect_param) ~= dim
    stk_error('The parameter and the Gaussian log-prior have incompatible dimension.',...
        'IncorrectSize');
end

% Calculate values
vect_param = vect_param - gauss_lp.mean;        % Centering
vect_param = (gauss_lp.eigenvect')*vect_param;  % Change the space

eigen_vals = gauss_lp.eigenvals;

 % derivative
dmlog_prior_dpar = vect_param./eigen_vals; 
dmlog_prior_dpar(isnan(dmlog_prior_dpar)) = 0;  % Correspond to the case,
% where eigen values is 0, and param == mean. In this case, return 0.
% If eigen values is 0, and param ~= mean, this will return +-Inf.
% If eigen values is +Inf, this will always return 0 (nor prior).

% minus log prior
mlog_prior = 0.5*(vect_param'*dmlog_prior_dpar);
if all(~isinf(eigen_vals)) && all(eigen_vals > 0)
    mlog_prior = mlog_prior + 0.5*dim*log(2*pi) + 0.5*log(prod(eigen_vals));
    % add the determinant only if all eigenvalues are different of 0
    % and +Inf
end

% Get back in initial space
dmlog_prior_dpar = (gauss_lp.eigenvect)*dmlog_prior_dpar;

end

