% CALL : [lb, ub] = stk_param_getdefaultbounds(gauss_lp)
% [overload stk_prior function]
%
% Return the bounds of optimization according to the Gaussian log-prior.
% In fact, it returns interval at 5-sigma of the prior.
%
function [lb, ub] = stk_param_getdefaultbounds(gauss_lp)

%% Parameter of the Gaussian
transMat = gauss_lp.eigenvect;
meanVect = gauss_lp.mean;
sigVal = sqrt(gauss_lp.eigenvals);

%% Calculate standard deviation matrix
sigVal(isinf(sigVal)) = 1i; % change the infinite values by i,
% in order to allow effect the multiplication
StdMat = transMat*sigVal;   % tranMat cannot have any imaginary values,
% so any imaginaty values in StdMat correspond to infinite

ind = abs(imag(StdMat)) < 1e-8; % normal value, imaginary part has 0
% <=> no infinite values
StdMat(ind) = real(StdMat(ind));
StdMat_to_inf = imag(StdMat(~ind));
StdMat(~ind) = sign(StdMat_to_inf).*Inf(size(StdMat_to_inf));
% get a non-zero imaginary part <=> infinite value

%% 5-sigma interval
StdMat_5 = 5*StdMat;

l = meanVect - StdMat_5;
u = meanVect + StdMat_5;

lb = min(l, u);
ub = max(l, u);
end

